{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {
    "tags": [
     "remove_cell"
    ]
   },
   "outputs": [],
   "source": [
    "from IPython.display import display, IFrame\n",
    "def example(url): display(IFrame(url, width=\"100%\", height=600))"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Додаток А. Рекомендації зі створення інтерфейсу\n",
    "## Обкладинка програми\n",
    "Кожна програма повинна мати «обкладинку» вигляду, представленого на рисунку А.1. У кожній програмі потрібно вказати потрібні номер і тему роботи, групу, прізвище та ініціали студента, рік виконання роботи, вибрати правильну форму написання слова «виконав» залежно від статі студента.\n",
    "\n",
    "![Обкладинка програми](../images/appA1.png)\n",
    " \n",
    "Рисунок А.1 – Обкладинка програми"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "```{margin}\n",
    "`conio.h` доступний лише у деяких компіляторах на Windows. На інших ОС варто використовувати бібліотеку [`ncurses`](https://invisible-island.net/ncurses/) або одну із бібліотек сумісності, наприклад [цю](https://sourceforge.net/projects/conioforlinux/) чи [цю](https://sourceforge.net/projects/linux-conioh/).\n",
    "\n",
    "Для спрощення розробки обкладинки ви можете скорситатися нашою простою бібліотекою, що дає кросплатформенний доступ до функцій `gotoxy` та `clrscr`: [`terminal.h`](https://gist.github.com/GasperPaul/68d4d9209aa9895aa89eefd09d418eec)\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Маніпуляції з консолью вимагають підключення додаткових бібліотек, що залежать від вашої операційної системи. Для Windows це може бути `conio.h` чи `windows.h`. На Linux та інших Unix-подібних системах традиційно використовують бібліотеку [`ncurses`](https://invisible-island.net/ncurses/).\n",
    "\n",
    "Для кросплатформенної розробки можна використовувати бібліотеку [`pdcurses`](https://github.com/wmcbrine/PDCurses).\n",
    "\n",
    "Для створення обкладинки потрібно:\n",
    "1. Підключити бібліотеку `stdio.h` та бібліотеку для маніпуляцій консолью.\n",
    "1. Очистити екран.\n",
    "1. Для креслення рамки використайте символи з такими кодами ASCII: \n",
    "  - 218 (лівий верхній кут),\n",
    "  - 196 (горизонтальна лінія),\n",
    "  - 191 (правий верхній кут),\n",
    "  - 179 (вертикальна лінія),\n",
    "  - 192 (лівий нижній кут),\n",
    "  - 217 (правий нижній кут).\n",
    "  \n",
    "  \n",
    "  | | | |\n",
    "  | --- | --- | --- |\n",
    "  | `┌` 218 | `─` 196 | `┐` 191 |\n",
    "  | `│` 179 |         | `│` 179 |\n",
    "  | `└` 192 | `─` 196 | `┘` 217 |\n",
    "  \n",
    "1. Використовуйте функцію `void gotoxy(int x, int y)`, яка поміщає курсор текстового екрана в точку з координатами `х, у`. (Стандартний розмір терміналу — 80 х 25.)\n",
    "1. Поставити очікування уведення будь-якого символу й вивести рядок, наприклад, `\"Для завершення натисніть Enter\"`.\n",
    "\n",
    "## Організація простого меню\n",
    "Майже в усіх завданнях користувачу надають вибір із декількох варіантів. Наприклад, для демонстрації роботи програм можна використовувати випадкові числа, а можна — числа, уведені з клавіатури. Вибір користувача часто реалізують у вигляді меню. Просте меню виглядає так:\n",
    "\n",
    "```\n",
    "Виберіть один із варіантів:\n",
    "1. Уведення чисел із клавіатури\n",
    "2. Генерація випадкових чисел\n",
    "3. Вихід із програми\n",
    "Уведіть номер вибраного пункту:\n",
    "```\n",
    "\n",
    "Розгляньмо використання меню на наступному прикладі."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "```{margin}\n",
    "Запустити цей приклад можна за [посиланням](https://repl.it/@GasperPaul/AppendixA-Example-A1).\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "#### Приклад А.1\n",
    "За трьома сторонами трикутника обчислити його площу.\n",
    "\n",
    "```c\n",
    "#include <stdio.h>\n",
    "#include <stdlib.h>\n",
    "#include <time.h> \n",
    "#include <math.h>\n",
    "\n",
    "void clrscr(void) \n",
    "{\n",
    "  system(\"cls||clear\");\n",
    "}\n",
    "\n",
    "void draw_menu(void)\n",
    "{\n",
    "  printf(\"Виберіть один із варіантів:\\n\");\n",
    "  printf(\"1. Уведення чисел із клавіатури\\n\");\n",
    "  printf(\"2. Генерація випадкових чисел\\n\");\n",
    "  printf(\"3. Вихід із програми\\n\");\n",
    "  printf(\"Уведіть номер вибраного пункту:\\n\");\n",
    "}\n",
    "\n",
    "void get_user_selection(char* ch)\n",
    "{\n",
    "  scanf(\" %c\", ch);\n",
    "  while(getchar() != '\\n'); // очищуємо буфер вводу\n",
    "}\n",
    "\n",
    "_Bool is_valid_input(char ch)\n",
    "{\n",
    "  return ch == '1' || ch == '2' || ch == '3';\n",
    "}\n",
    "\n",
    "int main()\n",
    "{\n",
    "  float a, b, c; /*сторони трикутника*/\n",
    "  float p;       /*напівпериметр трикутника*/\n",
    "  float s;       /*площа трикутника*/\n",
    "\n",
    "  clrscr();\n",
    "\n",
    "  /*нескінченний цикл*/\n",
    "  while(1) {\n",
    "    draw_menu();\n",
    "\n",
    "    char ch; /*обраний пункт меню*/\n",
    "    get_user_selection(&ch);\n",
    "\n",
    "    /*якщо увели неправильний параметр – знову меню*/\n",
    "    if (!is_valid_input(ch)) {\n",
    "      clrscr();\n",
    "      printf(\"Ви увели неправильний номер.\\n\\n\");\n",
    "      continue;\n",
    "    }\n",
    "\n",
    "    switch (ch)\n",
    "    {\n",
    "      case '1': /*уведення з клавіатури*/\n",
    "        printf(\"Уведіть сторони трикутника:\\n\");\n",
    "        printf(\"a = \");\n",
    "        scanf(\"%f\", &a);\n",
    "        printf(\"b = \");\n",
    "        scanf(\"%f\", &b);\n",
    "        printf(\"c = \");\n",
    "        scanf(\"%f\", &c);\n",
    "        break;\n",
    "      case '2': /*генерація випадкових значень*/\n",
    "        srand(time(NULL));\n",
    "        /*для згенерованих чисел виконується умова:\n",
    "        будь-яка зі сторін менша за суму двох інших*/\n",
    "        a = 6 + rand() % 5; //a = [6..10]\n",
    "        b = 6 + rand() % 5; //b = [6..10]\n",
    "        c = 6 + rand() % 5; //c = [6..10]\n",
    "        break;\n",
    "      case '3': /*вихід із програми*/\n",
    "        exit(0);\n",
    "        break;\n",
    "    }\n",
    "\n",
    "    p = (a + b + c) / 2;\n",
    "    s = sqrt(p * (p-a) * (p-b) * (p-c));\n",
    "    printf(\"Площа трикутника S = %f\\n\\n\", s);\n",
    "  }\n",
    "\n",
    "  return 0;\n",
    "}\n",
    "```"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.9.7"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
